@@ -1,13 +1,27 @@
+"""
+Custom defined RedVox Cloud derived errors.
+"""
+
+
 class CloudApiError(Exception):
-    def __init__(self, message: str = ""):
-        super().__init__(message)
+    """
+    A generic Cloud API error.
+    """
 
 
 class AuthenticationError(CloudApiError):
+    """
+    A cloud authentication error.
+    """
+
     def __init__(self):
         super().__init__("Authentication Error")
 
 
 class ApiConnectionError(CloudApiError):
+    """
+    A connection error.
+    """
+
     def __init__(self, message: str = ""):
         super().__init__(f"ApiConnectionError: {message}")
