@@ -2,24 +2,20 @@
 Read Redvox data from a single directory
 Data files can be either API 900 or API 1000 data formats
 """
-from typing import List, Optional, Tuple
-from datetime import timedelta
+from typing import List, Optional
+from datetime import timedelta, datetime
 import multiprocessing
 import multiprocessing.pool
-from itertools import repeat
-from multiprocessing import cpu_count, Manager, Process, Queue
-import queue
 
 import pyarrow as pa
+import psutil
 
 import redvox.settings as settings
 import redvox.api1000.proto.redvox_api_m_pb2 as api_m
-from redvox.common import offset_model
-from redvox.common import api_conversions as ac
-from redvox.common import io
-from redvox.common import file_statistics as fs
+from redvox.common import offset_model, io,\
+    api_conversions as ac,\
+    file_statistics as fs
 from redvox.common.parallel_utils import maybe_parallel_map
-from redvox.common.station_old import Station as StationOld
 from redvox.common.station import Station
 from redvox.common.errors import RedVoxExceptions
 
@@ -33,18 +29,27 @@ meta_py_stct = pa.struct([("api", pa.float64()), ("sub_api", pa.float64()), ("ma
                           ])
 
 
+PERCENT_FREE_MEM_USE = .8  # Percentage of total free memory to use when creating stations (1. is 100%)
+
+
 class ApiReader:
     """
     Reads data from api 900 or api 1000 format, converting all data read into RedvoxPacketM for
-        ease of comparison and use.
+    ease of comparison and use.
+
     Properties:
         filter: io.ReadFilter with the station ids, start and end time, start and end time padding, and
-                types of files to read
+        types of files to read
+
         base_dir: str of the directory containing all the files to read
+
         structured_dir: bool, if True, the base_dir contains a specific directory structure used by the
-                        respective api formats.  If False, base_dir only has the data files.  Default False.
+        respective api formats.  If False, base_dir only has the data files.  Default False.
+
         files_index: io.Index of the files that match the filter that are in base_dir
+
         index_summary: io.IndexSummary of the filtered data
+
         debug: bool, if True, output additional information during function execution.  Default False.
     """
 
@@ -81,6 +86,24 @@ class ApiReader:
         self.errors = RedVoxExceptions("APIReader")
         self.files_index = self._get_all_files(_pool)
         self.index_summary = io.IndexSummary.from_index(self._flatten_files_index())
+        if len(self.files_index) > 0:
+            mem_split_factor = len(self.files_index) if settings.is_parallelism_enabled() else 1
+            self.chunk_limit = psutil.virtual_memory().available * PERCENT_FREE_MEM_USE / mem_split_factor
+            max_file_size = max([fe.decompressed_file_size_bytes for fi in self.files_index for fe in fi.entries])
+            if max_file_size > self.chunk_limit:
+                raise MemoryError(f"System requires {max_file_size} bytes of memory to process a file but only has "
+                                  f"{self.chunk_limit} available.  Please free or add more RAM.")
+            elif max_file_size * sum([len(fi.entries) for fi in self.files_index]) > self.chunk_limit:
+                raise MemoryError("TOO MUCH DATA DAWG")
+            if debug:
+                if mem_split_factor == 1:
+                    print(f"{len(self.files_index)} stations have {int(self.chunk_limit)} "
+                          f"bytes for loading files in memory.")
+                else:
+                    print(f"{mem_split_factor} stations each have "
+                          f"{int(self.chunk_limit)} bytes for loading files in memory.")
+        else:
+            self.chunk_limit = 0
 
         if debug:
             self.errors.print()
@@ -145,6 +168,29 @@ class ApiReader:
             _pool.close()
         return index
 
+    def _redo_index(self, station_ids: set, new_start: datetime, new_end: datetime):
+        """
+        Redo the index for files using new start and end dates.  removes any buffer time at the start and end of the
+        new query.  Returns the updated index or None
+
+        :param station_ids: set of ids to get
+        :param new_start: new start time to get data from
+        :param new_end: new end time to get data from
+        :return: Updated index or None
+        """
+        diff_s = diff_e = timedelta(seconds=0)
+        new_index = self._apply_filter(io.ReadFilter()
+                                       .with_start_dt(new_start)
+                                       .with_end_dt(new_end)
+                                       .with_extensions(self.filter.extensions)
+                                       .with_api_versions(self.filter.api_versions)
+                                       .with_station_ids(station_ids)
+                                       .with_start_dt_buf(diff_s)
+                                       .with_end_dt_buf(diff_e))
+        if len(new_index.entries) > 0:
+            return new_index
+        return None
+
     def _check_station_stats(
             self,
             station_index: io.Index,
@@ -175,43 +221,29 @@ class ApiReader:
         if timing_offsets is None:
             return [station_index]
 
-        diff_s = diff_e = timedelta(seconds=0)
-
         # if our filtered files do not encompass the request even when the packet times are updated
         # try getting 1.5 times the difference of the expected start/end and the start/end of the data
         insufficient_str = ""
-        if self.filter.start_dt and timing_offsets.adjusted_start > self.filter.start_dt:
+        if (self.filter.start_dt and timing_offsets.adjusted_start > self.filter.start_dt) or \
+                (self.filter.end_dt and timing_offsets.adjusted_start >= self.filter.end_dt):
             insufficient_str += f" {self.filter.start_dt} (start)"
-            # diff_s = self.filter.start_dt_buf + 1.5 * (timing_offsets.adjusted_start - self.filter.start_dt)
             new_end = self.filter.start_dt - self.filter.start_dt_buf
             new_start = new_end - 1.5 * (timing_offsets.adjusted_start - self.filter.start_dt)
-            new_index = self._apply_filter(io.ReadFilter()
-                                           .with_start_dt(new_start)
-                                           .with_end_dt(new_end)
-                                           .with_extensions(self.filter.extensions)
-                                           .with_api_versions(self.filter.api_versions)
-                                           .with_station_ids(set(station_index.summarize().station_ids()))
-                                           .with_start_dt_buf(diff_s)
-                                           .with_end_dt_buf(diff_e))
-            if len(new_index.entries) > 0:
+            new_index = self._redo_index(set(station_index.summarize().station_ids()), new_start, new_end)
+            if new_index:
                 station_index.append(new_index.entries)
                 stats.extend(fs.extract_stats(new_index))
-        if self.filter.end_dt and timing_offsets.adjusted_end < self.filter.end_dt:
+
+        if (self.filter.end_dt and timing_offsets.adjusted_end < self.filter.end_dt) or \
+                (self.filter.start_dt and timing_offsets.adjusted_end <= self.filter.start_dt):
             insufficient_str += f" {self.filter.end_dt} (end)"
-            # diff_e = self.filter.end_dt_buf + 1.5 * (self.filter.end_dt - timing_offsets.adjusted_end)
             new_start = self.filter.end_dt + self.filter.end_dt_buf
             new_end = new_start + 1.5 * (self.filter.end_dt - timing_offsets.adjusted_end)
-            new_index = self._apply_filter(io.ReadFilter()
-                                           .with_start_dt(new_start)
-                                           .with_end_dt(new_end)
-                                           .with_extensions(self.filter.extensions)
-                                           .with_api_versions(self.filter.api_versions)
-                                           .with_station_ids(set(station_index.summarize().station_ids()))
-                                           .with_start_dt_buf(diff_s)
-                                           .with_end_dt_buf(diff_e))
-            if len(new_index.entries) > 0:
+            new_index = self._redo_index(set(station_index.summarize().station_ids()), new_start, new_end)
+            if new_index:
                 station_index.append(new_index.entries)
                 stats.extend(fs.extract_stats(new_index))
+
         if len(insufficient_str) > 0:
             self.errors.append(f"Data for {station_index.summarize().station_ids()} exists, "
                                f"but not at:{insufficient_str}")
@@ -229,6 +261,28 @@ class ApiReader:
 
         return list(results.values())
 
+    def _split_workload(self, findex: io.Index) -> List[io.Index]:
+        """
+        takes an index and splits it into chunks based on a size limit
+        while running_total + next_file_size < limit, adds files to a chunk (Index)
+        if limit is exceeded, adds the chunk and puts the next file into a new chunk
+
+        :param findex: index of files to split
+        :return: list of Index to process
+        """
+        packet_list = []
+        chunk_queue = 0
+        chunk_list = []
+        for f in findex.entries:
+            chunk_queue += f.decompressed_file_size_bytes
+            if chunk_queue > self.chunk_limit:
+                packet_list.append(io.Index(chunk_list))
+                chunk_queue = 0
+                chunk_list = []
+            chunk_list.append(f)
+        packet_list.append(io.Index(chunk_list))
+        return packet_list
+
     @staticmethod
     def read_files_in_index(indexf: io.Index) -> List[api_m.RedvoxPacketM]:
         """
@@ -292,31 +346,31 @@ class ApiReader:
 
         return result
 
-    def _stations_by_index(self, findex: io.Index) -> StationOld:
+    def _station_by_index(self, findex: io.Index) -> Station:
         """
         :param findex: index with files to build a station with
         :return: Station built from files in findex
         """
-        return StationOld(self.read_files_in_index(findex))
+        return Station.create_from_packets(self.read_files_in_index(findex))
 
-    def get_stations(self, pool: Optional[multiprocessing.pool.Pool] = None) -> List[StationOld]:
+    def get_stations(self, pool: Optional[multiprocessing.pool.Pool] = None) -> List[Station]:
         """
         :param pool: optional multiprocessing pool
         :return: List of all stations in the ApiReader
         """
         return list(maybe_parallel_map(pool,
-                                       self._stations_by_index,
+                                       self._station_by_index,
                                        self.files_index,
                                        chunk_size=1
                                        )
                     )
 
-    def get_station_by_id(self, get_id: str) -> Optional[List[StationOld]]:
+    def get_station_by_id(self, get_id: str) -> Optional[List[Station]]:
         """
         :param get_id: the id to filter on
         :return: list of all stations with the requested id or None if id can't be found
         """
-        result = [s for s in self.get_stations() if s.id == get_id]
+        result = [s for s in self.get_stations() if s.id() == get_id]
         if len(result) < 1:
             return None
         return result
