@@ -87,6 +87,12 @@ class EventOrigin:
         self.altitude_std = alt_std
         self.event_radius_m = event_radius_m
 
+    def __repr__(self):
+        return str(self.as_dict())
+
+    def __str__(self):
+        return str(self.as_dict())
+
     def as_dict(self) -> Dict:
         """
         :return: self as dict
@@ -181,6 +187,37 @@ class DataWindowConfig:
         self.use_model_correction = use_model_correction
         self.copy_edge_points = copy_edge_points
 
+    def __repr__(self):
+        return f"input_dir: {self.input_dir}, " \
+               f"structured_layout: {self.structured_layout}, " \
+               f"start_datetime: {self.start_datetime.__repr__()}, " \
+               f"end_datetime: {self.end_datetime.__repr__()}, " \
+               f"start_buffer_td: {self.start_buffer_td.__repr__()}, " \
+               f"end_buffer_td: {self.end_buffer_td.__repr__()}, " \
+               f"drop_time_s: {self.drop_time_s}, " \
+               f"station_ids: {list(self.station_ids) if self.station_ids else []}, " \
+               f"extensions: {list(self.extensions) if self.extensions else []}, " \
+               f"api_versions: {[a_v.value for a_v in self.api_versions] if self.api_versions else []}, " \
+               f"apply_correction: {self.apply_correction}, " \
+               f"use_model_correction: {self.use_model_correction}, " \
+               f"copy_edge_points: {self.copy_edge_points.value}"
+
+    def __str__(self):
+        return f"input_dir: {self.input_dir}, " \
+               f"structured_layout: {self.structured_layout}, " \
+               f"start_datetime: " \
+               f"{self.start_datetime.strftime('%Y-%m-%dT%H:%M:%S.%fZ') if self.start_datetime else None}, " \
+               f"end_datetime: {self.end_datetime.strftime('%Y-%m-%dT%H:%M:%S.%fZ') if self.end_datetime else None}, " \
+               f"start_buffer_td (in s): {self.start_buffer_td.total_seconds()}, " \
+               f"end_buffer_td (in s): {self.end_buffer_td.total_seconds()}, " \
+               f"drop_time_s: {self.drop_time_s}, " \
+               f"station_ids: {list(self.station_ids) if self.station_ids else []}, " \
+               f"extensions: {list(self.extensions) if self.extensions else []}, " \
+               f"api_versions: {[a_v.value for a_v in self.api_versions] if self.api_versions else []}, " \
+               f"apply_correction: {self.apply_correction}, " \
+               f"use_model_correction: {self.use_model_correction}, " \
+               f"copy_edge_points: {self.copy_edge_points.name}"
+
     def as_dict(self) -> Dict:
         return {"input_dir": self.input_dir,
                 "structured_layout": self.structured_layout,
@@ -191,7 +228,7 @@ class DataWindowConfig:
                 "drop_time_s": self.drop_time_s,
                 "station_ids": list(self.station_ids) if self.station_ids else [],
                 "extensions": list(self.extensions) if self.extensions else [],
-                "api_versions": [a_v.value for a_v in self.api_versions],
+                "api_versions": [a_v.value for a_v in self.api_versions] if self.api_versions else [],
                 "apply_correction": self.apply_correction,
                 "use_model_correction": self.use_model_correction,
                 "copy_edge_points": self.copy_edge_points.value
@@ -242,7 +279,7 @@ class DataWindow:
             event_name: str = "dw",
             event_origin: Optional[EventOrigin] = None,
             config: Optional[DataWindowConfig] = None,
-            out_dir: str = ".",
+            output_dir: str = ".",
             out_type: str = "NONE",
             make_runme: bool = False,
             debug: bool = False,
@@ -255,23 +292,20 @@ class DataWindow:
                                 origin event.  Default empty EventOrigin (no valid data)
         :param config: Optional DataWindowConfig which describes how to extract data from Redvox files.
                         Default None
-        :param out_dir: output directory for saving files.  Default "." (current directory)
+        :param output_dir: output directory for saving files.  Default "." (current directory)
         :param out_type: type of file to save the DataWindow as.  Options: "PARQUET", "LZ4", "NONE".
                             Default "NONE" (no saving)
         :param make_runme: if True, saves an example runme.py file with the data.  Default False
         :param debug: if True, outputs additional information during initialization.  Default False
         """
         self.event_name: str = event_name
-        if event_origin:
-            self.event_origin: EventOrigin = event_origin
-        else:
-            self.event_origin = EventOrigin()
-        self._fs_writer = dw_io.DataWindowFileSystemWriter(self.event_name, out_type, out_dir, make_runme)
+        self.event_origin: EventOrigin = event_origin if event_origin else EventOrigin()
+        self._fs_writer = dw_io.DataWindowFileSystemWriter(self.event_name, out_type, output_dir, make_runme)
         self.debug: bool = debug
         self._sdk_version: str = redvox.VERSION
         self._errors = RedVoxExceptions("DataWindow")
         self._stations: List[Station] = []
-        self.config = config
+        self._config = config
         if config:
             if config.start_datetime and config.end_datetime and (config.end_datetime <= config.start_datetime):
                 self._errors.append("DataWindow will not work when end datetime is before or equal to start datetime.\n"
@@ -281,13 +315,32 @@ class DataWindow:
         if self.debug:
             self.print_errors()
 
+    def __repr__(self):
+        return f"event_name: {self.event_name}, " \
+               f"event_origin: {self.event_origin.__repr__()}, " \
+               f"config: {self._config.__repr__()}, " \
+               f"output_dir: {os.path.abspath(self.save_dir())}, " \
+               f"out_type: {self._fs_writer.file_extension}, " \
+               f"make_runme: {self._fs_writer.make_run_me}, " \
+               f"sdk_version: {self._sdk_version}, " \
+               f"debug: {self.debug}"
+
+    def __str__(self):
+        return f"event_name: {self.event_name}, " \
+               f"event_origin: {self.event_origin.__str__()}, " \
+               f"config: {self._config.__str__()}, " \
+               f"output_dir: {os.path.abspath(self.save_dir())}, " \
+               f"out_type: {self._fs_writer.file_extension}, " \
+               f"make_runme: {self._fs_writer.make_run_me}, " \
+               f"sdk_version: {self._sdk_version}, " \
+               f"debug: {self.debug}"
+        # "stations": [s.__str__() for s in self._stations],
+
     def save_dir(self) -> str:
         """
-        :return: directory data is saved to
+        :return: directory data is saved to (empty string means saving to memory)
         """
-        if self._fs_writer.save_to_disk:
-            return self._fs_writer.save_dir()
-        return ""
+        return self._fs_writer.save_dir()
 
     def set_save_dir(self, new_save_dir: Optional[str] = "."):
         """
@@ -307,9 +360,9 @@ class DataWindow:
         """
         self._fs_writer.make_run_me = make_runme
 
-    def fs_writer(self) -> io.FileSystemWriter:
+    def fs_writer(self) -> dw_io.DataWindowFileSystemWriter:
         """
-        :return: FileSystemWriter for DataWindow
+        :return: DataWindowFileSystemWriter for DataWindow
         """
         return self._fs_writer
 
@@ -321,11 +374,11 @@ class DataWindow:
 
     def set_out_type(self, new_out_type: str):
         """
-        set the output type of the DataWindow.  options are "NONE", "PARQUET" and "LZ4"
+        set the output type of the DataWindow.  options are "NONE", "PARQUET" and "LZ4".  invalid values become "NONE"
 
         :param new_out_type: new output type of the DataWindow
         """
-        self._fs_writer.file_extension = new_out_type
+        self._fs_writer.set_extension(new_out_type)
 
     def as_dict(self) -> Dict:
         """
@@ -337,7 +390,7 @@ class DataWindow:
                 "end_time": self.end_date(),
                 "base_dir": self.save_dir(),
                 "stations": [s.default_station_json_file_name() for s in self._stations],
-                "config": self.config.as_dict(),
+                "config": self._config.as_dict(),
                 "debug": self.debug,
                 "errors": self._errors.as_dict(),
                 "sdk_version": self._sdk_version,
@@ -394,6 +447,7 @@ class DataWindow:
     def serialize(self, compression_factor: int = 4) -> Path:
         """
         Serializes and compresses this DataWindow to a file.
+        Uses the event_name and out_dir to name the file.
 
         :param compression_factor: A value between 1 and 12. Higher values provide better compression, but take
         longer. (default=4).
@@ -445,9 +499,9 @@ class DataWindow:
                 dwin = DataWindow(json_dict["event_name"], EventOrigin.from_dict(json_dict["event_origin"]),
                                   None, json_dict["base_dir"], json_dict["out_type"], json_dict["make_runme"],
                                   json_dict["debug"])
-                dwin.config = DataWindowConfig.from_dict(json_dict["config"])
-                dwin.errors = RedVoxExceptions.from_dict(json_dict["errors"])
-                dwin.set_sdk_version(json_dict["sdk_version"])
+                dwin._config = DataWindowConfig.from_dict(json_dict["config"])
+                dwin._errors = RedVoxExceptions.from_dict(json_dict["errors"])
+                dwin._sdk_version = json_dict["sdk_version"]
                 for st in json_dict["stations"]:
                     dwin.add_station(Station.from_json_file(os.path.join(json_dict["base_dir"], st), f"{st}.json"))
             elif out_type == dw_io.DataWindowOutputType.LZ4:
@@ -459,11 +513,13 @@ class DataWindow:
 
     def save(self) -> Path:
         """
-        save the DataWindow
-        :return: the path to where the files exist
+        save the DataWindow to disk if saving is enabled
+        if saving is not enabled, adds an error to the DataWindow and returns an empty path.
+
+        :return: the path to where the files exist; an empty path means no files were saved
         """
-        if self._fs_writer.save_to_disk:
-            if self._fs_writer.make_run_me:
+        if self._fs_writer.is_save_disk():
+            if self._fs_writer.is_use_disk() and self._fs_writer.make_run_me:
                 shutil.copyfile(os.path.abspath(inspect.getfile(run_me)),
                                 os.path.join(self._fs_writer.save_dir(), "runme.py"))
             if self._fs_writer.file_extension == "parquet":
@@ -473,6 +529,7 @@ class DataWindow:
         else:
             self._errors.append("Saving not enabled.")
             print("WARNING: Cannot save data window without knowing extension.")
+            return Path()
 
     @staticmethod
     def load(file_path: str) -> "DataWindow":
@@ -482,7 +539,17 @@ class DataWindow:
         :param file_path: full path of file to load
         :return: DataWindow from json metadata
         """
-        return DataWindow.from_json_dict(dw_io.json_file_to_data_window(file_path))
+        cur_path = os.getcwd()
+        os.chdir(os.path.dirname(file_path))
+        result = DataWindow.from_json_dict(dw_io.json_file_to_data_window(file_path))
+        os.chdir(cur_path)
+        return result
+
+    def config(self) -> DataWindowConfig:
+        """
+        :return: settings used to create the DataWindow
+        """
+        return self._config
 
     def sdk_version(self) -> str:
         """
@@ -573,7 +640,12 @@ class DataWindow:
         :return: first station matching params; if no params given, gets first station in list.
                     returns None if no station with given station_id exists.
         """
-        if station_id:
+        if len(self._stations) < 1:
+            self._errors.append(f"Attempted to get a station, but there are no stations in the data window!")
+            if self.debug:
+                print(f"Attempted to get a station, but there are no stations in the data window!")
+            return None
+        elif station_id:
             result = [s for s in self._stations if s.get_key().check_key(station_id, None, None)]
             if len(result) > 0:
                 return result[0]
@@ -581,8 +653,7 @@ class DataWindow:
             if self.debug:
                 print(f"Attempted to get station {station_id}, but that station is not in this data window!")
             return None
-        else:
-            return self._stations[0]
+        return self._stations[0]
 
     def get_station(self, station_id: str, station_uuid: Optional[str] = None,
                     start_timestamp: Optional[float] = None) -> Optional[List[Station]]:
@@ -603,9 +674,9 @@ class DataWindow:
             print(f"Attempted to get station {station_id}, but that station is not in this data window!")
         return None
 
-    def _add_sensor_to_window(self, station: Station):
+    # def _add_sensor_to_window(self, station: Station):
         # set the window start and end if they were specified, otherwise use the bounds of the data
-        self.create_window_in_sensors(station, self.config.start_datetime, self.config.end_datetime)
+        # self.create_window_in_sensors(station, self._config.start_datetime, self._config.end_datetime)
 
     def create_data_window(self, pool: Optional[multiprocessing.pool.Pool] = None):
         """
@@ -617,46 +688,52 @@ class DataWindow:
         _pool: multiprocessing.pool.Pool = multiprocessing.Pool() if pool is None else pool
 
         r_f = io.ReadFilter()
-        if self.config.start_datetime:
-            r_f.with_start_dt(self.config.start_datetime)
-        if self.config.end_datetime:
-            r_f.with_end_dt(self.config.end_datetime)
-        if self.config.station_ids:
-            r_f.with_station_ids(self.config.station_ids)
-        if self.config.extensions:
-            r_f.with_extensions(self.config.extensions)
+        if self._config.start_datetime:
+            r_f.with_start_dt(self._config.start_datetime)
+        if self._config.end_datetime:
+            r_f.with_end_dt(self._config.end_datetime)
+        if self._config.station_ids:
+            r_f.with_station_ids(self._config.station_ids)
+        if self._config.extensions:
+            r_f.with_extensions(self._config.extensions)
         else:
-            self.config.extensions = r_f.extensions
-        if self.config.api_versions:
-            r_f.with_api_versions(self.config.api_versions)
+            self._config.extensions = r_f.extensions
+        if self._config.api_versions:
+            r_f.with_api_versions(self._config.api_versions)
         else:
-            self.config.api_versions = r_f.api_versions
-        r_f.with_start_dt_buf(self.config.start_buffer_td)
-        r_f.with_end_dt_buf(self.config.end_buffer_td)
+            self._config.api_versions = r_f.api_versions
+        r_f.with_start_dt_buf(self._config.start_buffer_td)
+        r_f.with_end_dt_buf(self._config.end_buffer_td)
+
+        if self.debug:
+            print("Reading files from disk.  This may take a few minutes to complete.")
 
         # get the data to convert into a window
-        a_r = ApiReaderDw(self.config.input_dir, self.config.structured_layout, r_f,
-                          correct_timestamps=self.config.apply_correction,
-                          use_model_correction=self.config.use_model_correction,
+        a_r = ApiReaderDw(self._config.input_dir, self._config.structured_layout, r_f,
+                          correct_timestamps=self._config.apply_correction,
+                          use_model_correction=self._config.use_model_correction,
                           dw_base_dir=self.save_dir(),
-                          save_files=self._fs_writer.save_to_disk,
+                          dw_save_mode=self._fs_writer.save_mode(),
                           debug=self.debug, pool=_pool)
 
-        self._errors.extend_error(a_r.errors)
+        # self._errors.extend_error(a_r.errors)
+
+        if self._fs_writer.is_use_mem() and a_r.dw_save_mode != self._fs_writer.save_mode():
+            if self.debug:
+                print("Estimated size of files exceeds available memory.")
+                print("Automatically using temporary directory to store data.")
+            self._fs_writer.set_use_temp(True)
 
         # Parallel update
         # Apply timing correction in parallel by station
         sts = a_r.get_stations()
         if self.debug:
             print("num stations loaded: ", len(sts))
-        if self.config.apply_correction:
-            for st in maybe_parallel_map(_pool, Station.update_timestamps,
-                                         iter(sts), chunk_size=1):
-                self._add_sensor_to_window(st)
-                if self.debug:
-                    print("station processed: ", st.id())
-        else:
-            [self._add_sensor_to_window(s) for s in sts]
+
+        for st in maybe_parallel_map(_pool, Station.update_timestamps, iter(sts), chunk_size=1):
+            self.create_window_in_sensors(st, self._config.start_datetime, self._config.end_datetime)
+            if self.debug:
+                print("station processed: ", st.id())
 
         # check for stations without data
         self._check_for_audio()
@@ -668,12 +745,12 @@ class DataWindow:
 
         # must update the start and end in order for the data to be saved
         # update remaining data window values if they're still default
-        if not self.config.start_datetime and len(self._stations) > 0:
-            self.config.start_datetime = dtu.datetime_from_epoch_microseconds_utc(
+        if not self._config.start_datetime and len(self._stations) > 0:
+            self._config.start_datetime = dtu.datetime_from_epoch_microseconds_utc(
                 np.min([t.first_data_timestamp() for t in self._stations]))
         # end_datetime is non-inclusive, so it must be greater than our latest timestamp
-        if not self.config.end_datetime and len(self._stations) > 0:
-            self.config.end_datetime = dtu.datetime_from_epoch_microseconds_utc(
+        if not self._config.end_datetime and len(self._stations) > 0:
+            self._config.end_datetime = dtu.datetime_from_epoch_microseconds_utc(
                 np.max([t.last_data_timestamp() for t in self._stations]) + 1)
 
         # If the pool was created by this function, then it needs to managed by this function.
@@ -687,9 +764,9 @@ class DataWindow:
         remove = []
         for s in self._stations:
             if not s.has_audio_sensor():
-                remove.append(s.id())
+                remove.append(s.get_key())
         if len(remove) > 0:
-            self._stations = [s for s in self._stations if s.id() not in remove]
+            self._stations = [s for s in self._stations if s.get_key() not in remove]
 
     def _check_valid_ids(self):
         """
@@ -697,15 +774,15 @@ class DataWindow:
         and creates an error message for each id requested but has no data
         if there are no stations, creates a single error message declaring no data found
         """
-        if len(self._stations) < 1:
-            if len(self.config.station_ids) > 1:
-                add_ids = f"for all stations {self.config.station_ids} "
+        if len(self._stations) < 1 and self._config.station_ids:
+            if len(self._config.station_ids) > 1:
+                add_ids = f"for all stations {self._config.station_ids} "
             else:
                 add_ids = ""
-            self._errors.append(f"No data matching criteria {add_ids}in {self.config.input_dir}"
+            self._errors.append(f"No data matching criteria {add_ids}in {self._config.input_dir}"
                                 f"\nPlease adjust parameters of DataWindow")
-        elif len(self.station_ids()) > 0 and self.config.station_ids:
-            for ids in self.config.station_ids:
+        elif len(self._stations) > 0 and self._config.station_ids:
+            for ids in self._config.station_ids:
                 if ids.zfill(10) not in [i.id() for i in self._stations]:
                     self._errors.append(
                         f"Requested {ids} but there is no data to read for that station"
@@ -734,15 +811,19 @@ class DataWindow:
         else:
             end_datetime = dtu.datetime_to_epoch_microseconds_utc(dtu.datetime.max)
         self.process_sensor(station.audio_sensor(), station.id(), start_datetime, end_datetime)
-        for sensor in [s for s in station.data() if s.type() != SensorType.AUDIO]:
-            self.process_sensor(sensor, station.id(), station.audio_sensor().first_data_timestamp(),
-                                station.audio_sensor().last_data_timestamp())
-        # recalculate metadata
-        station.update_first_and_last_data_timestamps()
-        station.set_packet_metadata([meta for meta in station.packet_metadata()
-                                     if meta.packet_start_mach_timestamp < station.last_data_timestamp() and
-                                     meta.packet_end_mach_timestamp >= station.first_data_timestamp()])
-        self._stations.append(station)
+        if station.has_audio_data():
+            for sensor in [s for s in station.data() if s.type() != SensorType.AUDIO]:
+                self.process_sensor(sensor, station.id(), station.audio_sensor().first_data_timestamp(),
+                                    station.audio_sensor().last_data_timestamp())
+            # recalculate metadata
+            station.update_first_and_last_data_timestamps()
+            station.set_packet_metadata([meta for meta in station.packet_metadata()
+                                         if meta.packet_start_mach_timestamp < station.last_data_timestamp() and
+                                         meta.packet_end_mach_timestamp >= station.first_data_timestamp()])
+            if self._fs_writer.is_save_disk():
+                station.set_save_mode(io.FileSystemSaveMode.DISK)
+                station.set_save_dir(self.save_dir() if self._fs_writer.is_use_disk() else self._fs_writer.get_temp())
+            self._stations.append(station)
 
     def process_sensor(self, sensor: SensorData, station_id: str, start_date_timestamp: float,
                        end_date_timestamp: float):
@@ -775,9 +856,12 @@ class DataWindow:
             # check if all the samples have been cut off
             is_audio = sensor.type() == SensorType.AUDIO
             if end_index <= start_index:
+                self._errors.append(
+                    f"Data window for {station_id} {'Audio' if is_audio else sensor.type().name} "
+                    f"sensor has truncated all data points"
+                )
                 if is_audio:
-                    self._errors.append(f"Data window for {station_id} "
-                                        f"Audio sensor has truncated all data points")
+                    sensor.empty_data_table()
                 elif last_before_start is not None and first_after_end is None:
                     first_entry = sensor.pyarrow_table().slice(last_before_start, 1).to_pydict()
                     first_entry["timestamps"] = [start_date_timestamp]
@@ -789,19 +873,14 @@ class DataWindow:
                 elif last_before_start is not None and first_after_end is not None:
                     sensor.write_pyarrow_table(
                         sensor.interpolate(start_date_timestamp, last_before_start, 1,
-                                           self.config.copy_edge_points == gpu.DataPointCreationMode.COPY))
-                else:
-                    self._errors.append(
-                        f"Data window for {station_id} {sensor.type().name} "
-                        f"sensor has truncated all data points"
-                    )
+                                           self._config.copy_edge_points == gpu.DataPointCreationMode.COPY))
             else:
                 _arrow = sensor.pyarrow_table().slice(start_index, end_index-start_index)
                 # if sensor is audio or location, we want nan'd edge points
                 if sensor.type() in [SensorType.LOCATION, SensorType.AUDIO]:
                     new_point_mode = gpu.DataPointCreationMode.NAN
                 else:
-                    new_point_mode = self.config.copy_edge_points
+                    new_point_mode = self._config.copy_edge_points
                 # add in the data points at the edges of the window if there are defined start and/or end times
                 slice_start = _arrow["timestamps"].to_numpy()[0]
                 slice_end = _arrow["timestamps"].to_numpy()[-1]
@@ -813,14 +892,14 @@ class DataWindow:
                 else:
                     end_sample_interval = dtu.seconds_to_microseconds(sensor.sample_interval_s())
                     start_sample_interval = -end_sample_interval
-                    if self.config.end_datetime:
-                        end_samples_to_add = int((dtu.datetime_to_epoch_microseconds_utc(self.config.end_datetime)
+                    if self._config.end_datetime:
+                        end_samples_to_add = int((dtu.datetime_to_epoch_microseconds_utc(self._config.end_datetime)
                                                   - slice_end) / end_sample_interval)
                     else:
                         end_samples_to_add = 0
-                    if self.config.start_datetime:
+                    if self._config.start_datetime:
                         start_samples_to_add = int((slice_start -
-                                                    dtu.datetime_to_epoch_microseconds_utc(self.config.start_datetime))
+                                                    dtu.datetime_to_epoch_microseconds_utc(self._config.start_datetime))
                                                    / end_sample_interval)
                     else:
                         start_samples_to_add = 0
