@@ -1,3 +1,10 @@
+"""
+This file is designed to show the usage of the redvox SDK when loading DataWindows from existing files
+This file is expected to be in the same directory as the JSON file being loaded,
+and there is only one JSON file to load.
+This file is expected to work with SDK version 3.1.4 and higher
+"""
+
 import redvox.common.data_window as dwa
 from redvox.common.io import get_json_file
 
@@ -11,7 +18,7 @@ def main():
     print("Stations by ID in Data Window:", [sttn.id() for sttn in dw.stations()])
     st = dw.first_station()
 
-    print(f"Sensors in Station {st.id()}:", st.get_sensors())
+    print(f"Sensors in Station {st.id()}:", st.get_sensor_names())
     audio = st.audio_sensor()
     print(f"Audio Sensor {audio.name} Data: ", audio.get_microphone_data())
 
