@@ -3,7 +3,7 @@ Defines generic sensor data and data for API-independent analysis
 all timestamps are integers in microseconds unless otherwise stated
 """
 import enum
-from typing import List, Union, Dict, Optional, Tuple
+from typing import List, Dict, Optional, Tuple
 from pathlib import Path
 import os
 
@@ -16,10 +16,10 @@ import pyarrow.parquet as pq
 
 import redvox.common.sensor_io as io
 import redvox.common.date_time_utils as dtu
-from redvox.common.io import FileSystemWriter as Fsw
+from redvox.common.io import FileSystemSaveMode, FileSystemWriter as Fsw
 from redvox.common import offset_model as om
 from redvox.common.errors import RedVoxExceptions
-from redvox.common.gap_and_pad_utils import calc_evenly_sampled_timestamps
+from redvox.common.gap_and_pad_utils import calc_evenly_sampled_timestamps, AudioWithGaps
 from redvox.api1000.wrapped_redvox_packet.station_information import (
     NetworkType,
     PowerState,
@@ -31,11 +31,19 @@ from redvox.api1000.wrapped_redvox_packet.sensors.location import LocationProvid
 from redvox.api1000.wrapped_redvox_packet.sensors.image import ImageCodec
 from redvox.api1000.wrapped_redvox_packet.sensors.audio import AudioCodec
 
+# function used to translate values of enumerated columns
+COLUMN_TO_ENUM_FN = {"location_provider": lambda l: LocationProvider(l).name,
+                     "image_codec": lambda l: ImageCodec(l).name,
+                     "audio_codec": lambda l: AudioCodec(l).name,
+                     "network_type": lambda l: NetworkType(l).name,
+                     "power_state": lambda l: PowerState(l).name,
+                     "cell_service": lambda l: CellServiceState(l).name,
+                     "wifi_wake_lock": lambda l: WifiWakeLock(l).name,
+                     "screen_state": lambda l: ScreenState(l).name}
 # columns that cannot be interpolated
 NON_INTERPOLATED_COLUMNS = ["compressed_audio", "image"]
 # columns that are not numeric but can be interpolated
-NON_NUMERIC_COLUMNS = ["location_provider", "image_codec", "audio_codec", "network_type",
-                       "power_state", "cell_service", "wifi_wake_lock", "screen_state"]
+NON_NUMERIC_COLUMNS = list(COLUMN_TO_ENUM_FN.keys())
 
 
 class SensorType(enum.Enum):
@@ -172,7 +180,8 @@ class SensorData:
             save_data: bool = False,
             base_dir: str = ".",
             gaps: Optional[List[Tuple[float, float]]] = None,
-            show_errors: bool = False
+            show_errors: bool = False,
+            use_temp_dir: bool = False
     ):
         """
         initialize the sensor data with params
@@ -200,6 +209,7 @@ class SensorData:
                         the pairs of points exists to maintain sample rate and are not considered valid points.
                         Default None
         :param show_errors: if True, show any errors encountered.  Default False
+        :param use_temp_dir: if True, use a temp directory to save data.  Default False
         """
         self._errors: RedVoxExceptions = RedVoxExceptions("Sensor")
         self.name: str = sensor_name
@@ -210,25 +220,55 @@ class SensorData:
         self._is_sample_rate_fixed: bool = is_sample_rate_fixed
         self._timestamps_altered: bool = are_timestamps_altered
         self._use_offset_model: bool = use_offset_model_for_correction
-        self._fs_writer = Fsw("", "parquet", base_dir, save_data)
+        if save_data:
+            save_mode = FileSystemSaveMode.DISK
+        elif use_temp_dir:
+            save_mode = FileSystemSaveMode.TEMP
+        else:
+            save_mode = FileSystemSaveMode.MEM
+        self._fs_writer = Fsw("", "parquet", base_dir, save_mode)
         self._gaps: List[Tuple] = gaps if gaps else []
-        self._data: Optional[pa.Table()] = None
-        if sensor_data:
+        set_data_as_sensor_data = True
+        if sensor_data is not None:
             if "timestamps" not in sensor_data.schema.names:
                 self._errors.append('must have a column titled "timestamps"')
             elif sensor_data['timestamps'].length() > 0:
-                self.set_file_name(f"{sensor_type.name}_{int(sensor_data['timestamps'][0].as_py())}")
-                if calculate_stats:
+                set_data_as_sensor_data = False
+                if calculate_stats and np.isnan(sample_interval_s) and np.isnan(sample_rate_hz) \
+                        and np.isnan(sample_interval_std_s):
                     self.organize_and_update_stats(sensor_data)
                 elif sensor_data["timestamps"].length() > 1:
                     self.sort_by_data_timestamps(sensor_data)
                 else:
                     self.write_pyarrow_table(sensor_data)
-        else:
-            self._errors.append('cannot be empty')
+        if set_data_as_sensor_data:
+            self._data = sensor_data
         if show_errors:
             self.print_errors()
 
+    def __repr__(self):
+        return f"name: {self.name}, " \
+               f"type: {self._type.value}, " \
+               f"num_samples: {self.num_samples()}, " \
+               f"sample_rate_hz: {self._sample_rate_hz}, " \
+               f"sample_interval_s: {self._sample_interval_s}, " \
+               f"sample_interval_std_s: {self._sample_interval_std_s}, " \
+               f"is_sample_rate_fixed: {self._is_sample_rate_fixed}, " \
+               f"timestamps_altered: {self._timestamps_altered}, " \
+               f"use_offset_model: {self._use_offset_model}, " \
+               f"gaps: {[g for g in self._gaps]}"
+
+    def __str__(self):
+        return f"name: {self.name}, " \
+               f"type: {self._type.name}, " \
+               f"num_samples: {self.num_samples()}, " \
+               f"sample_rate_hz: {self._sample_rate_hz}, " \
+               f"sample_interval_s: {self._sample_interval_s}, " \
+               f"sample_interval_std_s: {self._sample_interval_std_s}, " \
+               f"is_sample_rate_fixed: {self._is_sample_rate_fixed}, " \
+               f"timestamps_altered: {self._timestamps_altered}, " \
+               f"use_offset_model: {self._use_offset_model}"
+
     @staticmethod
     def from_dir(
             sensor_name: str,
@@ -241,7 +281,8 @@ class SensorData:
             are_timestamps_altered: bool = False,
             calculate_stats: bool = False,
             use_offset_model_for_correction: bool = False,
-            save_data: bool = False) -> "SensorData":
+            save_data: bool = False,
+            use_temp_dir: bool = False) -> "SensorData":
         """
         init but with a path to directory containing parquet file(s) instead of a table of data
 
@@ -259,14 +300,14 @@ class SensorData:
         :param use_offset_model_for_correction: if True, use an offset model to correct timestamps, otherwise
                                                 use the best known offset.  default False
         :param save_data: if True, save the data of the sensor to disk, otherwise use a temporary dir.  default False
+        :param use_temp_dir: if True, save the data using a temporary directory.  default False
         :return: RedvoxSensor object
         """
         result = SensorData(sensor_name,
                             ds.dataset(data_path, format="parquet", exclude_invalid_files=True).to_table(),
                             sensor_type, sample_rate_hz, sample_interval_s, sample_interval_std_s,
                             is_sample_rate_fixed, are_timestamps_altered, calculate_stats,
-                            use_offset_model_for_correction, save_data, data_path)
-        result.set_file_name()
+                            use_offset_model_for_correction, save_data, data_path, use_temp_dir=use_temp_dir)
         return result
 
     @staticmethod
@@ -283,6 +324,7 @@ class SensorData:
             use_offset_model_for_correction: bool = False,
             save_data: bool = False,
             arrow_dir: str = "",
+            use_temp_dir: bool = False
     ) -> "SensorData":
         """
         init but with a dictionary
@@ -304,11 +346,13 @@ class SensorData:
         :param save_data: if True, save the data of the sensor to disk, otherwise use a temporary dir.  default False
         :param arrow_dir: directory to save pyarrow table, default "" (current dir).  default temporary dir if not
                             saving data
+        :param use_temp_dir: If True, use a temporary directory to save the data.  default False
         :return: RedvoxSensor object
         """
         return SensorData(sensor_name, pa.Table.from_pydict(sensor_data), sensor_type, sample_rate_hz,
                           sample_interval_s, sample_interval_std_s, is_sample_rate_fixed, are_timestamps_altered,
-                          calculate_stats, use_offset_model_for_correction, save_data, arrow_dir)
+                          calculate_stats, use_offset_model_for_correction, save_data, arrow_dir,
+                          use_temp_dir=use_temp_dir)
 
     def save(self, file_name: Optional[str] = None) -> Optional[Path]:
         """
@@ -319,7 +363,7 @@ class SensorData:
                             [sensor_type]_[first_timestamp].json
         :return: The path to the saved file or None if unable to save.
         """
-        if self._fs_writer.save_to_disk:
+        if self._fs_writer.is_save_disk():
             return self.to_json_file(file_name)
         return None
 
@@ -336,17 +380,25 @@ class SensorData:
         else:
             return self.from_json_file(in_dir, file)
 
+    def set_save_mode(self, new_save_mode: FileSystemSaveMode):
+        """
+        changes the save mode to new_save_mode
+
+        :param new_save_mode: FileSystemSaveMode to change to
+        """
+        self._fs_writer.set_save_mode(new_save_mode)
+
     def is_save_to_disk(self) -> bool:
         """
         :return: True if sensor will be saved to disk
         """
-        return self._fs_writer.save_to_disk
+        return self._fs_writer.is_use_disk()
 
     def set_save_to_disk(self, save: bool):
         """
         :param save: If True, save to disk
         """
-        self._fs_writer.save_to_disk = save
+        self._fs_writer.set_use_disk(save)
 
     def set_file_name(self, new_file: Optional[str] = None):
         """
@@ -369,13 +421,13 @@ class SensorData:
         """
         return self._fs_writer.file_name
 
-    def set_save_dir(self, new_dir: Optional[str] = None):
+    def set_save_dir(self, new_dir: str = "."):
         """
         set the pyarrow directory or use the default: "." (current directory)
 
-        :param new_dir: the directory to change to; default None (use current directory)
+        :param new_dir: the directory to change to; default "." (use current directory)
         """
-        self._fs_writer.base_dir = new_dir if new_dir else "."
+        self._fs_writer.base_dir = new_dir
 
     def save_dir(self) -> str:
         """
@@ -395,9 +447,15 @@ class SensorData:
         """
         return self._fs_writer
 
+    def set_use_temp_dir(self, use_temp_dir: bool = False):
+        """
+        :param use_temp_dir: if True, use temp dir to save data.  default False
+        """
+        self._fs_writer.set_use_temp(use_temp_dir)
+
     def pyarrow_ds(self, base_dir: Optional[str] = None) -> ds.Dataset:
         """
-        :param base_dir: optional directory to use when loading the dataset.  if None, use self.base_dir()
+        :param base_dir: optional directory to use when loading the dataset.  if None, use self.save_dir()
         :return: the dataset stored in base_dir
         """
         if base_dir is None:
@@ -406,35 +464,65 @@ class SensorData:
 
     def pyarrow_table(self) -> pa.Table:
         """
-        :return: the table defined by the dataset stored in self
+        :return: the table defined by the _data property or the dataset stored in self.save_dir()
         """
-        return self._data if self._data else self.pyarrow_ds().to_table()
+        if self._data or self._fs_writer.is_use_mem():
+            return self._data
+        return self.pyarrow_ds().to_table()
 
     def data_df(self) -> pd.DataFrame:
         """
-        :return: the pandas dataframe defined by the dataset stored in self._arrow_dir
+        :return: the pandas dataframe defined by the dataset stored in self.save_dir()
         """
         return self.pyarrow_table().to_pandas()
 
-    def write_pyarrow_table(self, table: pa.Table):
+    def write_pyarrow_table(self, table: pa.Table, update_file_name: Optional[bool] = True):
         """
-        * saves the pyarrow table to disk or to memory.
+        saves the pyarrow table to disk or to memory.
+
+        * if there is no data or there is no column named timestamps in the table, an error will be created
         * if writing to disk, uses a default filename: {sensor_type}_{first_timestamp}.parquet
-        * creates the directory if it doesn't exist and removes any existing parquet files
+        * uses the directory defined by self.save_dir().  Creates the directory if it doesn't exist and removes any
+        existing parquet files from the directory if it exists
 
         :param table: the table to write
+        :param update_file_name: if True, updates the file name to match the new data.  Default True
         """
-        self._data = table
-        # self._fs_writer.create_dir()
-        # pq.write_table(table, self.full_path())
+        if table.num_rows < 1 or "timestamps" not in table.schema.names:
+            self._errors.append("Attempted to write invalid table.")
+        elif self._fs_writer.is_save_disk():
+            self._fs_writer.create_dir()
+            if update_file_name:
+                self.set_file_name(f"{self.type().name}_{int(table['timestamps'][0].as_py())}")
+            pq.write_table(table, self.full_path())
+            self._data = None
+        else:
+            self._data = table
 
-    def _actual_file_write_table(self):
+    def empty_data_table(self):
         """
-        file writing function; use when file system writing is dependable.
+        CAUTION: REMOVES ALL DATA AND COLUMNS FROM THE TABLE
         """
-        self._fs_writer.create_dir()
-        pq.write_table(self._data, self.full_path())
-        self._data = None
+        tbl = pa.Table.from_pydict({"timestamps": []})
+        if self._fs_writer.is_save_disk():
+            pq.write_table(tbl, self.full_path())
+            self._data = None
+        else:
+            self._data = tbl
+
+    def move_pyarrow_dir(self, new_dir: str) -> Path:
+        """
+        Move the sensor's pyarrow files to a new directory
+
+        :param new_dir: directory to save files into
+        """
+        old_sensor_save_dir = self.save_dir()
+        self.set_save_dir(os.path.join(new_dir, self._type.name))
+        for r, d, f in os.walk(old_sensor_save_dir):
+            for file in f:
+                self._fs_writer.create_dir()
+                os.rename(os.path.join(old_sensor_save_dir, file), os.path.join(self.save_dir(), file))
+        return Path(self.save_dir())
 
     def errors(self) -> RedVoxExceptions:
         """
@@ -510,9 +598,6 @@ class SensorData:
         else:
             order = "descending"
         data = pc.take(ptable, pc.sort_indices(ptable, sort_keys=[("timestamps", order)]))
-        # if not np.isnan(self.first_data_timestamp()) and self.first_data_timestamp() != data['timestamps'][0].as_py():
-        #     os.remove(self.full_path())
-        # self.set_file_name(f"{self._type.name}_{int(data['timestamps'][0].as_py())}")
         self.write_pyarrow_table(data)
 
     def organize_and_update_stats(self, ptable: pa.Table) -> "SensorData":
@@ -639,35 +724,26 @@ class SensorData:
         """
         :return: a list of the names of the columns (data channels) of the dataframe
         """
-        return self.pyarrow_table().schema.names
+        if self.pyarrow_table():
+            return self.pyarrow_table().schema.names
+        return []
 
-    def get_data_channel(self, channel_name: str) -> Union[np.array, List[str]]:
+    def get_data_channel(self, channel_name: str) -> np.array:
         """
         gets the data channel specified, raises an error and lists valid fields if channel_name is not in the dataframe
 
         :param channel_name: the name of the channel to get data for
         :return: the data values of the channel as a numpy array or list of strings for enumerated channels
         """
+        if not self.pyarrow_table():
+            self._errors.append(f"WARNING: There are no channels to access in this Sensor!")
+            return []
         _arrow = self.pyarrow_table()
         if channel_name not in _arrow.schema.names:
             self._errors.append(f"WARNING: {channel_name} does not exist; try one of {_arrow.schema.names}")
             return []
-        if channel_name == "location_provider":
-            return [LocationProvider(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "image_codec":
-            return [ImageCodec(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "audio_codec":
-            return [AudioCodec(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "network_type":
-            return [NetworkType(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "power_state":
-            return [PowerState(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "cell_service":
-            return [CellServiceState(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "wifi_wake_lock":
-            return [WifiWakeLock(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "screen_state":
-            return [ScreenState(c.as_py()).name for c in _arrow[channel_name]]
+        if channel_name in NON_NUMERIC_COLUMNS:
+            return np.array([COLUMN_TO_ENUM_FN[channel_name](c.as_py()) for c in _arrow[channel_name]])
         return _arrow[channel_name].to_numpy()
 
     def _get_non_numeric_data_channel(self, channel_name: str) -> List[str]:
@@ -677,23 +753,12 @@ class SensorData:
         :param channel_name: the name of the channel to get data for
         :return: the data values of the channel as a list of strings
         """
-        _arrow = self.pyarrow_table()
-        if channel_name == "location_provider":
-            return [LocationProvider(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "image_codec":
-            return [ImageCodec(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "audio_codec":
-            return [AudioCodec(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "network_type":
-            return [NetworkType(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "power_state":
-            return [PowerState(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "cell_service":
-            return [CellServiceState(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "wifi_wake_lock":
-            return [WifiWakeLock(c.as_py()).name for c in _arrow[channel_name]]
-        elif channel_name == "screen_state":
-            return [ScreenState(c.as_py()).name for c in _arrow[channel_name]]
+        if not self.pyarrow_table():
+            self._errors.append(f"WARNING: There are no channels to access in this Sensor!")
+        else:
+            _arrow = self.pyarrow_table()
+            if channel_name in NON_NUMERIC_COLUMNS:
+                return [COLUMN_TO_ENUM_FN[channel_name](c.as_py()) for c in _arrow[channel_name]]
         self._errors.append(f"WARNING: {channel_name} does not exist")
         return []
 
@@ -751,14 +816,14 @@ class SensorData:
                                                                  self._use_offset_model))
         # old_name = self.full_path()
         self.write_pyarrow_table(self.pyarrow_table().set_column(0, "timestamps", timestamps))
-        self.set_file_name()
+        # self.set_file_name()
         # os.rename(old_name, self.full_path())
         time_diffs = np.floor(np.diff(self.data_timestamps()))
         if len(time_diffs) > 1:
             self._sample_interval_s = dtu.microseconds_to_seconds(slope)
             if self._sample_interval_s > 0:
                 self._sample_rate_hz = 1 / self._sample_interval_s
-                self._sample_interval_std_s = dtu.microseconds_to_seconds(np.std(time_diffs))
+                self._sample_interval_std_s = dtu.microseconds_to_seconds(float(np.std(time_diffs)))
         self._timestamps_altered = True
 
     def interpolate(self, interpolate_timestamp: float, first_point: int, second_point: int = 0,
@@ -833,8 +898,7 @@ class SensorData:
                             [sensor_type]_[first_timestamp].json
         :return: path to json file
         """
-        if self._fs_writer.file_extension == "parquet":
-            self._actual_file_write_table()
+        self.write_pyarrow_table(self.pyarrow_table())
         return io.to_json_file(self, file_name)
 
     @staticmethod
@@ -859,6 +923,8 @@ class SensorData:
                                          json_data["sample_interval_std_s"], json_data["is_sample_rate_fixed"],
                                          json_data["timestamps_altered"], False, json_data["use_offset_model"])
             result.set_errors(RedVoxExceptions.from_dict(json_data["errors"]))
+            result.set_save_to_disk(True)
+            result.set_file_name()
             result.set_gaps(json_data["gaps"])
         else:
             result = SensorData("Empty")
@@ -871,42 +937,30 @@ class SensorData:
 
         :return: Self
         """
-        if self._type == SensorType.AUDIO:
-            self.__class__ = AudioSensor
-        if self._type == SensorType.COMPRESSED_AUDIO:
-            self.__class__ = CompressedAudioSensor
-        if self._type == SensorType.IMAGE:
-            self.__class__ = ImageSensor
-        if self._type == SensorType.LOCATION:
-            self.__class__ = LocationSensor
-        if self._type == SensorType.BEST_LOCATION:
-            self.__class__ = BestLocationSensor
-        if self._type == SensorType.STATION_HEALTH:
-            self.__class__ = StationHealthSensor
-        if self._type == SensorType.LIGHT:
-            self.__class__ = LightSensor
-        if self._type == SensorType.PRESSURE:
-            self.__class__ = PressureSensor
-        if self._type == SensorType.PROXIMITY or self._type == SensorType.INFRARED:
-            self.__class__ = ProximitySensor
-        if self._type == SensorType.RELATIVE_HUMIDITY:
-            self.__class__ = RelativeHumiditySensor
-        if self._type == SensorType.AMBIENT_TEMPERATURE:
-            self.__class__ = AmbientTemperatureSensor
-        if self._type == SensorType.ACCELEROMETER:
-            self.__class__ = AccelerometerSensor
-        if self._type == SensorType.GYROSCOPE:
-            self.__class__ = GyroscopeSensor
-        if self._type == SensorType.MAGNETOMETER:
-            self.__class__ = MagnetometerSensor
-        if self._type == SensorType.ORIENTATION:
-            self.__class__ = OrientationSensor
-        if self._type == SensorType.GRAVITY:
-            self.__class__ = GravitySensor
-        if self._type == SensorType.LINEAR_ACCELERATION:
-            self.__class__ = LinearAccelerationSensor
-        if self._type == SensorType.ROTATION_VECTOR:
-            self.__class__ = RotationVectorSensor
+        if self._type in SensorType:
+            sensor_class_from_type = {
+                SensorType.AUDIO: AudioSensor,
+                SensorType.COMPRESSED_AUDIO: CompressedAudioSensor,
+                SensorType.IMAGE: ImageSensor,
+                SensorType.LOCATION: LocationSensor,
+                SensorType.BEST_LOCATION: BestLocationSensor,
+                SensorType.STATION_HEALTH: StationHealthSensor,
+                SensorType.LIGHT: LightSensor,
+                SensorType.PRESSURE: PressureSensor,
+                SensorType.PROXIMITY: ProximitySensor,
+                SensorType.INFRARED: ProximitySensor,
+                SensorType.RELATIVE_HUMIDITY: RelativeHumiditySensor,
+                SensorType.AMBIENT_TEMPERATURE: AmbientTemperatureSensor,
+                SensorType.ACCELEROMETER: AccelerometerSensor,
+                SensorType.GYROSCOPE: GyroscopeSensor,
+                SensorType.MAGNETOMETER: MagnetometerSensor,
+                SensorType.ORIENTATION: OrientationSensor,
+                SensorType.GRAVITY: GravitySensor,
+                SensorType.LINEAR_ACCELERATION: LinearAccelerationSensor,
+                SensorType.ROTATION_VECTOR: RotationVectorSensor,
+                SensorType.UNKNOWN_SENSOR: SensorData
+            }
+            self.__class__ = sensor_class_from_type[self._type]
         return self
 
     def set_errors(self, errors: RedVoxExceptions):
@@ -950,10 +1004,50 @@ class AudioSensor(SensorData):
                  save_data: bool = False,
                  base_dir: str = ".",
                  gaps: Optional[List[Tuple[float, float]]] = None,
-                 show_errors: bool = False):
+                 show_errors: bool = False,
+                 use_temp_dir: bool = False):
         super().__init__(sensor_name, sensor_data, SensorType.AUDIO, sample_rate_hz, sample_interval_s,
                          sample_interval_std_s, is_sample_rate_fixed, are_timestamps_altered, calculate_stats,
-                         use_offset_model_for_correction, save_data, base_dir, gaps, show_errors)
+                         use_offset_model_for_correction, save_data, base_dir, gaps, show_errors, use_temp_dir)
+
+    @staticmethod
+    def from_metadata(sensor_name: str,
+                      data: AudioWithGaps,
+                      sample_rate_hz: float = np.nan,
+                      sample_interval_s: float = np.nan,
+                      sample_interval_std_s: float = np.nan,
+                      is_sample_rate_fixed: bool = False,
+                      are_timestamps_altered: bool = False,
+                      calculate_stats: bool = False,
+                      use_offset_model_for_correction: bool = False,
+                      save_data: bool = False,
+                      base_dir: str = ".",
+                      use_temp_dir: bool = False) -> "SensorData":
+        """
+        init but using metadata
+
+        :param sensor_name: name of the sensor
+        :param data: the metadata used to create the sensor
+        :param sample_rate_hz: sample rate in hz of the data, default np.nan
+        :param sample_interval_s: sample interval in seconds of the data, default np.nan
+        :param sample_interval_std_s: std dev of sample interval in seconds of the data, default np.nan
+        :param is_sample_rate_fixed: if True, sample rate is constant for all data, default False
+        :param are_timestamps_altered: if True, timestamps in the sensor have been altered from their
+                                        original values, default False
+        :param calculate_stats: if True, calculate sample_rate, sample_interval_s, and sample_interval_std_s
+                                default False
+        :param use_offset_model_for_correction: if True, use an offset model to correct timestamps, otherwise
+                                                use the best known offset.  default False
+        :param save_data: if True, save the data of the sensor to disk, otherwise use a temporary dir.  default False
+        :param base_dir: directory to save pyarrow table, default "." (current dir).  internally uses a temporary
+                            dir if not saving data
+        :param use_temp_dir: if True, save the data using a temporary directory.  default False
+        :return: RedvoxSensor object
+        """
+        return SensorData(sensor_name, data.create_timestamps(),
+                          SensorType.AUDIO, sample_rate_hz, sample_interval_s, sample_interval_std_s,
+                          is_sample_rate_fixed, are_timestamps_altered, calculate_stats,
+                          use_offset_model_for_correction, save_data, base_dir, use_temp_dir=use_temp_dir)
 
     def get_microphone_data(self) -> np.array:
         """
