@@ -10,21 +10,24 @@ import numpy as np
 from redvox.common.offset_model import OffsetModel
 from redvox.api1000.wrapped_redvox_packet.station_information import OsType
 from redvox.api1000.wrapped_redvox_packet.wrapped_packet import WrappedRedvoxPacketM
-
+from redvox.api1000.wrapped_redvox_packet.timing_information import TimingScoreMethod
+from redvox.common.errors import RedVoxExceptions
 # from redvox.api1000.wrapped_redvox_packet import event_streams as es
 import redvox.api1000.proto.redvox_api_m_pb2 as api_m
 
 
 def validate_station_key_list(
-        data_packets: List[api_m.RedvoxPacketM], debug: bool = False
+    data_packets: List[api_m.RedvoxPacketM], errors: RedVoxExceptions
 ) -> bool:
     """
     Checks for consistency in the data packets.  Returns False if discrepancies are found.
     If debug is True, will output the discrepancies.
+
     :param data_packets: list of WrappedRedvoxPacketM to look at
-    :param debug: bool, if True, output any discrepancies found, default False
+    :param errors: RedVoxExceptions detailing errors found while validating
     :return: True if no discrepancies found.  False otherwise
     """
+    my_errors = RedVoxExceptions("StationKeyValidation")
     if len(data_packets) < 2:
         return True
     j: np.ndarray = np.transpose(
@@ -65,13 +68,15 @@ def validate_station_key_list(
     for key, value in k.items():
         result = np.unique(value)
         if len(result) > 1:
-            if debug:
-                print(
-                    f"WARNING from station_utils: {data_packets[0].station_information.id} "
-                    f"{key} contains multiple unique values: {result}.\n"
-                    "Please update your query to focus on one of these values."
-                )
-            return False  # stop processing, one bad key is enough
+            my_errors.append(
+                f"WARNING: {data_packets[0].station_information.id} "
+                f"{key} contains multiple unique values: {result}.\n"
+                "Please update your query to focus on one of these values."
+            )
+
+    if my_errors.get_num_errors() > 0:
+        errors.extend_error(my_errors)
+        return False
 
     return True  # if here, everything is consistent
 
@@ -105,6 +110,7 @@ class StationKey:
         check if the key has the values specified.  If the parameter is None, any value will match.
         Note that NAN is a valid value for start_timestamps, but any station with start_timestamp = NAN
         will not match any value, including another NAN.
+
         :param station_id: station id, default None
         :param station_uuid: station uuid, default None
         :param start_timestamp: station start timestamp in microseconds since UTC epoch, default None
@@ -126,13 +132,13 @@ class StationKey:
     def compare_key(self, other_key: "StationKey") -> bool:
         """
         compare key to another station's key
+
         :param other_key: another station's key
         :return: True if the keys match
         """
         return self.check_key(other_key.id, other_key.uuid, other_key.start_timestamp_micros)
 
 
-# todo add event streams?
 class StationMetadata:
     """
     A container for all the packet metadata consistent across all packets
@@ -154,6 +160,7 @@ class StationMetadata:
     def __init__(self, app: str, packet: Optional[api_m.RedvoxPacketM] = None):
         """
         initialize the metadata
+
         :param app: app name
         :param packet: Optional WrappedRedvoxPacketM to read data from
         """
@@ -164,7 +171,7 @@ class StationMetadata:
             self.sub_api = packet.sub_api
             self.make = packet.station_information.make
             self.model = packet.station_information.model
-            self.os = packet.station_information.os
+            self.os: OsType = OsType(packet.station_information.os)
             self.os_version = packet.station_information.os_version
             self.app_version = packet.station_information.app_version
             self.is_private = packet.station_information.is_private
@@ -178,13 +185,39 @@ class StationMetadata:
             self.sub_api = np.nan
             self.make = ""
             self.model = ""
-            self.os = OsType.UNKNOWN_OS
+            self.os: OsType = OsType["UNKNOWN_OS"]
             self.os_version = ""
             self.app_version = ""
             self.is_private = False
             self.packet_duration_s = np.nan
             self.station_description = ""
 
+    def __repr__(self):
+        return f"app: {self.app}, " \
+               f"api: {self.api}, " \
+               f"sub_api: {self.sub_api}, " \
+               f"make: {self.make}, " \
+               f"model: {self.model}, " \
+               f"os: {self.os.value}, " \
+               f"os_version: {self.os_version}, " \
+               f"app_version: {self.app_version}, " \
+               f"is_private: {self.is_private}, " \
+               f"packet_duration_s: {self.packet_duration_s}, " \
+               f"station_description: {self.station_description}"
+
+    def __str__(self):
+        return f"app: {self.app}, " \
+               f"api: {self.api}, " \
+               f"sub_api: {self.sub_api}, " \
+               f"make: {self.make}, " \
+               f"model: {self.model}, " \
+               f"os: {self.os.name}, " \
+               f"os_version: {self.os_version}, " \
+               f"app_version: {self.app_version}, " \
+               f"is_private: {self.is_private}, " \
+               f"packet_duration_s: {self.packet_duration_s}, " \
+               f"station_description: {self.station_description}"
+
     def validate_metadata(self, other_metadata: "StationMetadata") -> bool:
         """
         :param other_metadata: another StationMetadata object to compare
@@ -204,6 +237,45 @@ class StationMetadata:
                 and self.station_description == other_metadata.station_description
         )
 
+    def as_dict(self) -> dict:
+        """
+        :return: dictionary representation of the metadata
+        """
+        return {
+            "app": self.app,
+            "api": self.api,
+            "sub_api": self.sub_api,
+            "make": self.make,
+            "model": self.model,
+            "os": self.os.name,
+            "os_version": self.os_version,
+            "app_version": self.app_version,
+            "is_private": self.is_private,
+            "packet_duration_s": self.packet_duration_s,
+            "station_description": self.station_description,
+            "other_metadata": self.other_metadata
+        }
+
+    @staticmethod
+    def from_dict(md_dict: dict) -> "StationMetadata":
+        """
+        :param md_dict: metadata dictionary
+        :return: StationMetadata from dictionary
+        """
+        result = StationMetadata(md_dict["app"])
+        result.api = md_dict["api"]
+        result.sub_api = md_dict["sub_api"]
+        result.make = md_dict["make"]
+        result.model = md_dict["model"]
+        result.os = OsType[md_dict["os"]]
+        result.os_version = md_dict["os_version"]
+        result.app_version = md_dict["app_version"]
+        result.is_private = md_dict["is_private"]
+        result.packet_duration_s = md_dict["packet_duration_s"]
+        result.station_description = md_dict["station_description"]
+        result.other_metadata = md_dict["other_metadata"]
+        return result
+
 
 class StationMetadataWrapped:
     """
@@ -226,6 +298,7 @@ class StationMetadataWrapped:
     def __init__(self, app: str, packet: Optional[WrappedRedvoxPacketM] = None):
         """
         initialize the metadata
+
         :param app: app name
         :param packet: Optional WrappedRedvoxPacketM to read data from
         """
@@ -279,18 +352,32 @@ class StationMetadataWrapped:
 class StationPacketMetadata:
     """
     A container for all the packet metadata that isn't consistent across all packets
+
     Properties:
-        packet_start_mach_timestamp: float, machine timestamp of packet start in microseconds since epoch UTC
-        packet_end_mach_timestamp: float, machine timestamp of packet end in microseconds since epoch UTC
-        packet_start_os_timestamp: float, os timestamp of packet start in microseconds since epoch UTC
-        packet_end_os_timestamp: float, os timestamp of packet end in microseconds since epoch UTC
-        timing_info_score: float, quality of timing information
-        other_metadata: dict, str: str of other metadata from the packet
+        packet_start_mach_timestamp: float, machine timestamp of packet start in microseconds since epoch UTC,
+        default np.nan
+
+        packet_end_mach_timestamp: float, machine timestamp of packet end in microseconds since epoch UTC,
+        default np.nan
+
+        packet_start_os_timestamp: float, os timestamp of packet start in microseconds since epoch UTC, default np.nan
+
+        packet_end_os_timestamp: float, os timestamp of packet end in microseconds since epoch UTC, default np.nan
+
+        server_packet_received_timestamp: float, timestamp from server when packet was received in
+        microseconds since epoch UTC
+
+        timing_info_score: float, quality of timing information, default np.nan
+
+        timing_score_method: TimingScoreMethod, method used to determine timing score, default "UNKNOWN"
+
+        other_metadata: dict, str: str of other metadata from the packet, default empty list
     """
 
     def __init__(self, packet: Optional[api_m.RedvoxPacketM] = None):
         """
         initialize the metadata
+
         :param packet: Optional WrappedRedvoxPacketM to read data from
         """
         self.other_metadata = {}
@@ -307,17 +394,40 @@ class StationPacketMetadata:
             self.packet_end_os_timestamp = (
                 packet.timing_information.packet_end_os_timestamp
             )
+            self.server_packet_receive_timestamp = packet.timing_information.server_acquisition_arrival_timestamp
             self.timing_info_score = packet.timing_information.score
+            self.timing_score_method = TimingScoreMethod(packet.timing_information.score_method)
         else:
             self.packet_start_mach_timestamp = np.nan
             self.packet_end_mach_timestamp = np.nan
             self.packet_start_os_timestamp = np.nan
             self.packet_end_os_timestamp = np.nan
+            self.server_packet_receive_timestamp = np.nan
             self.timing_info_score = np.nan
+            self.timing_score_method = TimingScoreMethod["UNKNOWN"]
+
+    def __repr__(self):
+        return f"packet_start_mach_timestamp: {self.packet_start_mach_timestamp}, " \
+               f"packet_end_mach_timestamp: {self.packet_end_mach_timestamp}, " \
+               f"packet_start_os_timestamp: {self.packet_start_os_timestamp}, " \
+               f"packet_end_os_timestamp: {self.packet_end_os_timestamp}, " \
+               f"server_packet_receive_timestamp: {self.server_packet_receive_timestamp}, " \
+               f"timing_info_score: {self.timing_info_score}, " \
+               f"timing_score_method: {self.timing_score_method.value}"
+
+    def __str__(self):
+        return f"packet_start_mach_timestamp: {self.packet_start_mach_timestamp}, " \
+               f"packet_end_mach_timestamp: {self.packet_end_mach_timestamp}, " \
+               f"packet_start_os_timestamp: {self.packet_start_os_timestamp}, " \
+               f"packet_end_os_timestamp: {self.packet_end_os_timestamp}, " \
+               f"server_packet_receive_timestamp: {self.server_packet_receive_timestamp}, " \
+               f"timing_info_score: {self.timing_info_score}, " \
+               f"timing_score_method: {self.timing_score_method.name}"
 
     def update_timestamps(self, om: OffsetModel, use_model_function: bool = True):
         """
         updates the timestamps in the metadata using the offset model
+
         :param om: OffsetModel to apply to data
         :param use_model_function: if True, use the offset model's correction function to correct time,
                                     otherwise use best offset (model's intercept value).  default True
@@ -327,6 +437,40 @@ class StationPacketMetadata:
         self.packet_start_os_timestamp = om.update_time(self.packet_start_os_timestamp, use_model_function)
         self.packet_end_os_timestamp = om.update_time(self.packet_end_os_timestamp, use_model_function)
 
+    def as_dict(self) -> dict:
+        """
+        :return: dictionary representation of the packet metadata
+        """
+        return {
+            "packet_start_mach_timestamp": self.packet_start_mach_timestamp,
+            "packet_end_mach_timestamp": self.packet_end_mach_timestamp,
+            "packet_start_os_timestamp": self.packet_start_os_timestamp,
+            "packet_end_os_timestamp": self.packet_end_os_timestamp,
+            "server_packet_receive_timestamp": self.server_packet_receive_timestamp,
+            "timing_info_score": self.timing_info_score,
+            "timing_score_method": self.timing_score_method.name,
+            "other_metadata": self.other_metadata
+        }
+
+    @staticmethod
+    def from_dict(pmd_dict: dict) -> "StationPacketMetadata":
+        """
+        :param pmd_dict: dictionary with StationPacketMetadata
+        :return: StationPacketMetadata from dictionary
+        """
+        result = StationPacketMetadata()
+        result.other_metadata = pmd_dict["other_metadata"]
+        result.packet_start_mach_timestamp = pmd_dict["packet_start_mach_timestamp"]
+        result.packet_end_mach_timestamp = pmd_dict["packet_end_mach_timestamp"]
+        result.packet_start_os_timestamp = pmd_dict["packet_start_os_timestamp"]
+        result.packet_end_os_timestamp = pmd_dict["packet_end_os_timestamp"]
+        result.server_packet_receive_timestamp = pmd_dict["server_packet_receive_timestamp"]
+        result.timing_info_score = pmd_dict["timing_info_score"]
+        result.timing_score_method = TimingScoreMethod[pmd_dict["timing_score_method"]
+                                                       if "timing_score_method" in pmd_dict.keys()
+                                                       else "UNKNOWN"]
+        return result
+
 
 class StationPacketMetadataWrapped:
     """
@@ -337,12 +481,14 @@ class StationPacketMetadataWrapped:
         packet_start_os_timestamp: float, os timestamp of packet start in microseconds since epoch UTC
         packet_end_os_timestamp: float, os timestamp of packet end in microseconds since epoch UTC
         timing_info_score: float, quality of timing information
+        timing_score_method: TimingScoreMethod, method used to determine timing score
         other_metadata: dict, str: str of other metadata from the packet
     """
 
     def __init__(self, packet: Optional[WrappedRedvoxPacketM] = None):
         """
         initialize the metadata
+
         :param packet: Optional WrappedRedvoxPacketM to read data from
         """
         self.other_metadata = {}
@@ -360,16 +506,19 @@ class StationPacketMetadataWrapped:
                 packet.get_timing_information().get_packet_end_os_timestamp()
             )
             self.timing_info_score = packet.get_timing_information().get_score()
+            self.timing_score_method = TimingScoreMethod(packet.get_timing_information().get_score_method())
         else:
             self.packet_start_mach_timestamp = np.nan
             self.packet_end_mach_timestamp = np.nan
             self.packet_start_os_timestamp = np.nan
             self.packet_end_os_timestamp = np.nan
             self.timing_info_score = np.nan
+            self.timing_score_method = TimingScoreMethod["UNKNOWN"]
 
     def update_timestamps(self, om: OffsetModel, use_model_function: bool = True):
         """
         updates the timestamps in the metadata using the offset model
+
         :param om: OffsetModel to apply to data
         :param use_model_function: if True, use the offset model's correction function to correct time,
                                     otherwise use best offset (model's intercept value).  default True
